@@ -3,6 +3,8 @@ mod convert_serial;
 mod coo;
 mod csc;
 mod csr;
+mod matrix_market;
 mod ops;
 mod pattern;
 mod proptest;
+mod test_data_examples;
